package be.iminds.ilabt.jfed.restapi.json;

import be.iminds.ilabt.jfed.restapi.json.util.JsonGeniUrnDeserializer;
import be.iminds.ilabt.jfed.restapi.json.util.JsonGeniUrnSerializer;
import be.iminds.ilabt.jfed.util.GeniUrn;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Project
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonIgnoreProperties("@context")
public class Project {
    private static final Logger LOG = LoggerFactory.getLogger(Project.class);

    private final String name;
    private final GeniUrn urn;

    @JsonCreator
    public Project(@JsonProperty("name") String name,
                   @JsonProperty("urn") GeniUrn urn) {
        this.name = name;
        this.urn = urn;
    }

    @JsonProperty
    public String getName() {
        return name;
    }

    @JsonProperty
    @JsonSerialize(using = JsonGeniUrnSerializer.class)
    @JsonDeserialize(using = JsonGeniUrnDeserializer.class)
    public GeniUrn getUrn() {
        return urn;
    }

    @JsonProperty(value="@type")
       public String getClassName() {
           return getClass().getSimpleName();
       }

       @JsonProperty(value="@type")
       private void setClassName(String className) {
           if (!getClass().getSimpleName().equals(className))
               throw new IllegalArgumentException("@type must be "+getClass().getSimpleName());
       }



    @Override
    public String toString() {
        ObjectMapper mapper = new ObjectMapper();
        try {
            return mapper.writerWithDefaultPrettyPrinter().writeValueAsString(this);
        } catch (JsonProcessingException e) {
            LOG.error("Exception converting "+getClass().getSimpleName()+" to JSON", e);
            return "Exception converting "+getClass().getSimpleName()+" to JSON: "+e.getMessage();
        }
    }
}
