package be.iminds.ilabt.jfed.restapi.debugdata;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;

public class ServerResourceMethodDDBuilder implements LogLineDDCollector, ClientApiCallDDCollector {
    private static final Logger LOG = LoggerFactory.getLogger(ServerResourceMethodDD.class);

    private Timestamp start;
    private Timestamp stop;
    private String method;
    private List<ClientApiCallDDBuilder> clientApiCalls;
    private List<LogLineDDBuilder> logLines;

    public ServerResourceMethodDDBuilder() {
    }

    public ServerResourceMethodDDBuilder(ServerResourceMethodDD orig) {
        this.start = orig.getStart();
        this.stop = orig.getStop();
        this.method = orig.getMethod();
        setClientApiCalls(orig.getClientApiCalls());
        setLogLines(orig.getLogLines());
    }

    public ServerResourceMethodDDBuilder setStart(Timestamp start) {
        this.start = start;
        return this;
    }

    public ServerResourceMethodDDBuilder setStop(Timestamp stop) {
        this.stop = stop;
        return this;
    }

    public ServerResourceMethodDDBuilder setMethod(String method) {
        this.method = method;
        return this;
    }

    @Override
    public ServerResourceMethodDDBuilder addClientApiCall(ClientApiCallDDBuilder clientApiCallDDBuilder) {
        if (this.clientApiCalls == null) {
            this.clientApiCalls = new ArrayList<>();
        }
        this.clientApiCalls.add(clientApiCallDDBuilder);
        return this;
    }
    public ServerResourceMethodDDBuilder setClientApiCalls(List<ClientApiCallDD> clientApiCalls) {
        if (clientApiCalls == null) {
            this.clientApiCalls = null;
        } else {
            this.clientApiCalls = new ArrayList<>();
            for (ClientApiCallDD dd : clientApiCalls) {
                this.clientApiCalls.add(new ClientApiCallDDBuilder(dd));
            }
        }
        return this;
    }


    @Override
    public ServerResourceMethodDDBuilder addLogLine(LogLineDDBuilder logLineDDBuilder) {
        if (this.logLines == null) {
            this.logLines = new ArrayList<>();
        }
        this.logLines.add(logLineDDBuilder);
        return this;
    }
    public ServerResourceMethodDDBuilder setLogLines(List<LogLineDD> logLines) {
        if (logLines == null) {
            this.logLines = null;
        } else {
            this.logLines = new ArrayList<>();
            for (LogLineDD dd : logLines) {
                this.logLines.add(new LogLineDDBuilder(dd));
            }
        }
        return this;
    }

    public ServerResourceMethodDD create() {
        return new ServerResourceMethodDD(
                start, stop, method,
                ClientApiCallDDBuilder.createAll(clientApiCalls),
                LogLineDDBuilder.createAll(logLines)
        );
    }

    public static List<ServerResourceMethodDD> createAll(List<ServerResourceMethodDDBuilder> builders) {
        if (builders == null)
            return null;

        List<ServerResourceMethodDD> res = new ArrayList<>();
        for (ServerResourceMethodDDBuilder b : builders) {
            res.add(b.create());
        }
        return res;
    }

    @Override
    public String toString() {
        ObjectMapper mapper = new ObjectMapper();
        try {
            return mapper.writerWithDefaultPrettyPrinter().writeValueAsString(create());
        } catch (JsonProcessingException e) {
            LOG.error("Exception converting "+getClass().getSimpleName()+" to JSON", e);
            return "Exception converting "+ getClass().getSimpleName() +" to JSON: "+e.getMessage();
        }
    }
}